# Submodules

| Name        | Purpose |
| :---------- | :--------------- |
| ASCII       | ASCII file formats (includes GeoCSV, SLIST, and variants)    |
| FastIO      | Replacement low-level I/O functions to avoid thread locking    |
| Quake       | Earthquake seismology                                               |
| RandSeis    | Generate SeisBase structures with quasi-random entries    |
| SEED        | Standard for the Exchange of Earthquake Data (SEED) file format    |
| SUDS        | Seismic Unified Data System (SUDS) file format    |
| SeisHDF     | Dedicated support for seismic HDF5 subformats    |
| UW          | University of Washington data format    |


## Using Submodules

At the Julia prompt, type *using SeisBase.NNNN* where NNNN is the submodule name; for example, `using SeisBase.Quake` loads the Quake submodule.


## RandSeis
This submodule is used to quickly generate SeisBase objects with quasi-random
field contents. Access it with "using SeisBase.RandSeis"

The following are true of all random data objects generated by the RandSeis
module:

* Channels have SEED-compliant IDs, sampling frequencies, and data types.
* Random junk fills :notes and :misc.
* Sampling frequency (:fs) is chosen from a set of common values.
* Channel data are randomly generated.
* Time gaps are automatically inserted into regularly-sampled data.

```@docs
SeisBase.RandSeis.randPhaseCat
SeisBase.RandSeis.randSeisChannel
SeisBase.RandSeis.randSeisData
SeisBase.RandSeis.randSeisEvent
SeisBase.RandSeis.randSeisHdr
SeisBase.RandSeis.randSeisSrc
```

## SEED
Submodule for the Standard for the Exchange of Earthquake Data (SEED) file format; includes additional functionality.

* [`SeisBase.SEED.dataless_support`](@ref)
* [`mseed_support`](@ref)
* [`seed_support`](@ref)

### Scanning SEED Volumes


```@docs
SeisBase.SEED.scan_seed
```

## UW
The UW submodule extends functionality for the University of Washington (UW) file format(s).

The UW data format was created in the 1970s by the Pacific Northwest Seismic Network (PNSN), USA, for event archival. It remained in use through the 1990s. A UW event is described by a pickfile and a corresponding data file, whose filenames were identical except for the last character. The data file is self-contained; the pick file is not required to read raw trace data. However, station locations were stored in an external text file.

Only UW-2 data files are supported by SeisBase. We have only seen UW-1 data files in Exabyte tapes from the 1980s.

```@docs
SeisBase.UW.uwpf
```
