import SeisBase
import SeisBase: get_svn

# =====================================================================
# Setup
@info("Please allow 20 minutes for all tests to execute.")
cd(dirname(pathof(SeisBase))*"/../test")
# if isdir("SampleFiles") == false
  # get_svn("https://github.com/jpjones76/SeisIO-TestData/trunk/SampleFiles", "SampleFiles")
# end
include("local_restricted.jl")
include("test_helpers.jl")

# Announce test begin
test_start  = Dates.now()
ltn         = 48
printstyled(stdout,
  string(test_start, ": tests begin, path = ", path, ", has_restricted = ",
    has_restricted, ", keep_log = ", keep_log, ", keep_samples = ",
    keep_samples, "\n"),
  color=:light_green,
  bold=true)

# =====================================================================
# Run all tests
# grep "include(joinpath" runtests.jl | awk -F "(" '{print $3}' | awk -F "," {'print $1'}

# notes on testing:
# The submodules "DataFormats," "SEED," and "Quake" are almost completely reliant
# on the (non-existent) sample files, so they have been removed from the suite of
# tests performed. For the other submodules, only a portion of the tests actually rely
# on the sample files. In such cases, I've commented out only the lines within the test
# files that require the sample files, so that the rest of the useful tests can be
# performed. A few specific files (in the variable "skip_files" below) are also
# dependent on the sample files, so we skip them in testing via the "if" statement at
# line 70. Overall, this is a bit complicated since multiple methods have been used to
# eliminate certain tests, but one should be able to begin the process of replacing
# the test files (if they so choose) by cloning a version of the SeisBase repo
# from before August 2023, running the tests, and addressing the errors one-by-one as
# they arise (which is how we ended up here- just replace the sample files and values used
# to test them instead of simply commenting out the offending lines as I did.) Good luck!
# -stephanie

# ======= modified files ========
# TestHelpers/2_constants.jl: 3-4
# CoreUtils/test_ls.jl: 6,44,53-55
# TestHelpers/breaking_seis.jl: 49-52
# Utils/test_units.jl: 4-21
# Processing/test_convert_seis.jl: 55-91
# Processing/test_merge.jl: 848-862
# Processing/test_resp.jl: 137,147-149
# Nodal/0_prelim.jl: 26,28
# Web/test_essential_web.jl: 75-133
# Web/test_fdsn.jl: 120-131

skip_files = ["test_guess.jl","test_native_io.jl","test_rescale.jl","test_ungap.jl",
              "test_nodal_types.jl","test_processing.jl","test_read_nodal.jl",
              "test_utils.jl"]
#for d in ["CoreUtils", "Types", "RandSeis", "Utils", "NativeIO", "DataFormats", "SEED", "Processing", "Nodal", "Quake", "Web"]
for d in ["CoreUtils", "Types", "RandSeis", "Utils", "NativeIO", "Processing", "Nodal","Web"]
  ld = length(d)
  ll = div(ltn - ld - 2, 2)
  lr = ll + (isodd(ld) ? 1 : 0)
  printstyled(string("="^ll, " ", d, " ", "="^lr, "\n"), color=:cyan, bold=true)
  for i in readdir(joinpath(path, d))
    if !(i in skip_files)
        f = joinpath(d,i)
        if endswith(i, ".jl")
          printstyled(lpad(" "*f, ltn)*"\n", color=:cyan)
          write(out, string("\n\ntest ", f, "\n\n"))
          flush(out)
          include(f)
        end
    end
  end
end

# =====================================================================
# Cleanup
include("cleanup.jl")
(keep_samples == true) || include("rm_samples.jl")
keep_log || safe_rm("runtests.log")

# Announce tests end
test_end = Dates.now()
δt = 0.001*(test_end-test_start).value
printstyled(string(test_end, ": tests end, elapsed time (mm:ss.μμμ) = ",
                   @sprintf("%02i", round(Int, div(δt, 60))), ":",
                   @sprintf("%06.3f", rem(δt, 60)), "\n"),
            color=:light_green,
            bold=true)
tut_file = realpath(path * "/../tutorial/install.jl")
ex_file = realpath(path * "/examples.jl")
printstyled("To run the interactive tutorial in a browser, execute: include(\"",
            tut_file, "\")\n", color=:cyan, bold=true)
printstyled("To run some data acquisition examples from the Julia prompt, ",
            "execute: include(\"", ex_file, "\")\n", color=:cyan)
